import os

from sqlalchemy import create_engine
from doris_alchemy import registry


def test_connect():
    print(registry)
    host = os.environ['DORIS_HOST']
    user = os.environ['DORIS_USER']
    password = os.environ['DORIS_PASSWORD']
    port = os.environ['DORIS_PORT']
    database = os.environ['DORIS_DATABASE']

    engine = create_engine(f"doris://{user}:{password}@{host}:{port}/{database}?charset=utf8mb4")
    assert engine.driver == 'mysqldb'
    with engine.connect() as conn:
        pass

    engine = create_engine(f"doris+pymysql://{user}:{password}@{host}:{port}/{database}?charset=utf8mb4")
    assert engine.driver == 'pymysql'

    with engine.connect() as conn:
        pass
    #
    engine = create_engine(f"doris+mysqldb://{user}:{password}@{host}:{port}/{database}?charset=utf8mb4")
    assert engine.driver == 'mysqldb'

    with engine.connect() as conn:
        pass



#! /usr/bin/python3

# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from abc import ABC, abstractmethod
import logging
import re
from typing import Iterable, Optional, List, Any, Sequence, Type, Dict
from sqlalchemy import Numeric, Integer, Float
from sqlalchemy.sql import sqltypes
from sqlalchemy.sql.type_api import TypeEngine

from doris_alchemy.util import ensure_sequence
from doris_alchemy.util import join_args_with_quote

logger = logging.getLogger(__name__)


class TINYINT(Integer):  # pylint: disable=no-init
    __visit_name__ = "TINYINT"


class LARGEINT(Integer):  # pylint: disable=no-init
    __visit_name__ = "LARGEINT"


class DOUBLE(Float):  # pylint: disable=no-init
    __visit_name__ = "DOUBLE"


class HLL(Numeric):  # pylint: disable=no-init
    __visit_name__ = "HLL"


class BITMAP(Numeric):  # pylint: disable=no-init
    __visit_name__ = "BITMAP"


class QUANTILE_STATE(Numeric):  # pylint: disable=no-init
    __visit_name__ = "QUANTILE_STATE"

class AGG_STATE(Numeric):  # pylint: disable=no-init
    __visit_name__ = "AGG_STATE"

class ARRAY(TypeEngine):  # pylint: disable=no-init
    __visit_name__ = "ARRAY"

    @TypeEngine.python_type.getter
    def python_type(self) -> Optional[Type[List[Any]]]:
        return list


class MAP(TypeEngine):  # pylint: disable=no-init
    __visit_name__ = "MAP"

    @TypeEngine.python_type.getter
    def python_type(self) -> Optional[Type[Dict[Any, Any]]]:
        return dict


class STRUCT(TypeEngine):  # pylint: disable=no-init
    __visit_name__ = "STRUCT"

    @TypeEngine.python_type.getter
    def python_type(self) -> Optional[Type[Any]]:
        return None


_type_map = {
    # === Boolean ===
    "boolean": sqltypes.BOOLEAN,
    # === Integer ===
    "tinyint": sqltypes.SMALLINT,
    "smallint": sqltypes.SMALLINT,
    "int": sqltypes.INTEGER,
    "bigint": sqltypes.BIGINT,
    "largeint": LARGEINT,
    # === Floating-point ===
    "float": sqltypes.FLOAT,
    "double": DOUBLE,
    # === Fixed-precision ===
    "decimal": sqltypes.DECIMAL,
    "decimalv3": sqltypes.DECIMAL,
    # === String ===
    "varchar": sqltypes.VARCHAR,
    "char": sqltypes.CHAR,
    "json": sqltypes.JSON,
    "jsonb": sqltypes.JSON,
    "text": sqltypes.TEXT,
    "string": sqltypes.String,
    # === Date and time ===
    "date": sqltypes.DATE,
    "datev2": sqltypes.DATE,
    "datetime": sqltypes.DATETIME,
    "datetimev2": sqltypes.DATETIME,
    # === Structural ===
    'array': ARRAY,
    'map': MAP,
    'struct': STRUCT,
    'hll': HLL,
    'quantile_state': QUANTILE_STATE,
    'bitmap': BITMAP,
    'agg_state': AGG_STATE,
}


def parse_sqltype(type_str: str) -> TypeEngine:
    type_str = type_str.strip().lower()
    match = re.match(r"^(?P<type>\w+)\s*(?:\((?P<options>.*)\))?", type_str)
    if not match:
        logger.warning(f"Could not parse type name '{type_str}'")
        return sqltypes.NULLTYPE
    type_name = match.group("type")

    if type_name not in _type_map:
        logger.warning(f"Did not recognize type '{type_name}'")
        return sqltypes.NULLTYPE
    type_class = _type_map[type_name]
    return type_class()


class RenderedMixin(ABC):
    @abstractmethod
    def render(self) -> str:
        pass


class HASH(RenderedMixin):
    def __init__(self, keys: Sequence[str]|str, buckets: Optional[int] = None):
        self.keys: Iterable[str] = ensure_sequence(keys)
        self.buckets = buckets or 'auto'

    def render(self) -> str:
        keys_str = 'HASH' + join_args_with_quote(*self.keys)
        buckets_str = f'BUCKETS {self.buckets}'
        return keys_str + ' ' + buckets_str


class RANGE(RenderedMixin):
    def __init__(self, keys: Sequence[str]|str, part_info: Sequence = tuple()):
        self.keys: Iterable[str] = ensure_sequence(keys)
        self.part_info = ensure_sequence(part_info)

    def render(self) -> str:
        keys_str = 'RANGE' + join_args_with_quote(*self.keys)
        if len(self.part_info) > 0:
            part_str = ',\n    '.join([str(val) for val in self.part_info])
            part_str = '(\n    ' + part_str + '\n)'
        else:
            part_str = '()'
        return keys_str + ' ' + part_str


class RANDOM(RenderedMixin):
    def __init__(self, buckets: Optional[int]=None):
        self.buckets = buckets or 'auto'

    def render(self) -> str:
        keys_str = 'RANDOM'
        keys_str += f' BUCKETS {self.buckets}'
        return keys_str



import os
import pytest

import sqlalchemy as sa
from sqlalchemy import create_engine

from doris_alchemy import datatype
from doris_alchemy.datatype import RANGE
from doris_alchemy.datatype import HASH


@pytest.fixture
def doris_engine():
    host = os.environ['DORIS_HOST']
    user = os.environ['DORIS_USER']
    password = os.environ['DORIS_PASSWORD']
    port = os.environ['DORIS_PORT']
    database = os.environ['DORIS_DATABASE']

    engine = create_engine(f"doris://{user}:{password}@{host}:{port}/{database}?charset=utf8mb4")

    return engine


def test_create_table(doris_engine):
    metadata_obj = sa.MetaData()
    table_obj = sa.Table(
        'test_table',
        metadata_obj,
        sa.Column("id", datatype.Integer),
        doris_unique_key=('id', ),
        doris_partition_by=RANGE('id'),
        doris_distributed_by=HASH('id'),
        doris_properties={"replication_allocation": "tag.location.default: 1"}
    )

    metadata_obj.create_all(doris_engine)

    metadata_obj.drop_all(doris_engine, [table_obj])



from typing import Any, Dict, Optional, Sequence, Tuple
from sqlalchemy import Engine, MetaData, String, Table, Text
from sqlalchemy.orm import DeclarativeBase
from doris_alchemy.datatype import HASH, RANGE
from doris_alchemy.datatype import RANDOM


METADATA = MetaData()


class DorisBase(DeclarativeBase):
    metadata = METADATA
    
    __table_args__:Dict[str, Any]
    __table_args__ = {
        'doris_properties': {"replication_allocation": "tag.location.default: 1"}
        }
    
    doris_distributed_by: HASH|RANDOM
    doris_partition_by: HASH|RANDOM|RANGE
    doris_properties: dict
    doris_autogen_primary_key: bool
    doris_unique_key: str|Sequence[str]
    doris_duplicate_key: str|Sequence[str]
    doris_aggregate_key: str|Sequence[str]
    
    type_annotation_map = {
        str: String().with_variant(Text, 'doris')
    }
    
    
    @classmethod
    def __base_table_args(cls) -> dict:
        if '__table_args__' not in DorisBase.__dict__:
            return {}
        table_args = DorisBase.__dict__['__table_args__']
        if isinstance(table_args, dict):
            return table_args
        return {}
    
    
    def __init_subclass__(cls, **kw: Any) -> None:
        if cls.__table_args__ is None:
            cls.__table_args__ = {}
        super_table_args = cls.__base_table_args()
        cls.__table_args__.update(super_table_args)
        # assert hasattr(cls, 'doris_distributed_by') or 'doris_distributed_by' in cls.__table_args__,\
        #     'You must define distributed_by for orm model. Via doris_distributed_by attribute, or __table_args__'
        if hasattr(cls, 'doris_distributed_by'):
            cls.__table_args__['doris_distributed_by'] = getattr(cls, 'doris_distributed_by')
        if hasattr(cls, 'doris_partition_by'):
            cls.__table_args__['doris_partition_by'] = getattr(cls, 'doris_partition_by')
        if hasattr(cls, 'doris_unique_key'):
            cls.__table_args__['doris_unique_key'] = getattr(cls, 'doris_unique_key')
        if hasattr(cls, 'doris_autogen_primary_key') and cls.doris_autogen_primary_key:
            cls.__table_args__['doris_autogen_primary_key'] = True
            
        super().__init_subclass__()
    
    
    def to_dict(self) -> dict[str, Any]:
        d = self.__dict__
        if '_sa_instance_state' in d:
            d.pop('_sa_instance_state')
        return d

    def __repr__(self) -> str:
        return f'{self.__class__.__name__}{self.to_dict()}'
    
    @classmethod
    def get_table(cls) -> Optional[Table]:
        tname = cls.__tablename__
        schema = cls.__table_args__.get('schema')
        if schema:
            tname = f'{schema}.{tname}'
        __mtd = cls.metadata
        if tname in __mtd.tables:
            return __mtd.tables[tname]
        return None
    
    @classmethod
    def create(cls, eng: Engine) -> None:
        t = cls.get_table()
        assert t is not None, f'Table {cls.__tablename__} is missing from Metadata!!'
        t.create(eng)
    
    
    @classmethod
    def drop(cls, eng: Engine) -> None:
        t = cls.get_table()
        assert t is not None, f'Table {cls.__tablename__} is missing from Metadata!!'
        t.drop(eng)


from datetime import datetime
import os
from pprint import pprint
from sqlalchemy import BigInteger, Column, DateTime, Engine, Integer, String, Table, Text, create_engine, insert, select, text
from sqlalchemy.orm import Mapped, mapped_column, Session

from doris_alchemy.datatype import HASH, RANGE
from doris_alchemy.datatype import RANDOM
from doris_alchemy.orm_base import METADATA, DorisBase

USER = os.environ['DORIS_PROD_USER']
PWD = os.environ['DORIS_PROD_PWD']
HOST = '10.0.100.115'
PORT = '9030'
DB = 'test'

def make_eng() -> Engine:
    return create_engine(f"doris+mysqldb://{USER}:{PWD}@{HOST}:{PORT}/{DB}")


table = Table(
    'dummy_table',
    METADATA,
    Column('id', Integer),
    Column('name', String(64), nullable=False),
    Column('description', Text),
    Column('date', DateTime),
    
    # doris_unique_key=('id'),
    doris_partition_by=RANGE('id'),
    # doris_distributed_by=HASH('id'),
    doris_distributed_by = RANDOM(32),
    doris_properties={"replication_allocation": "tag.location.default: 1"},
    doris_autogen_primary_key=True
)


class Dummy(DorisBase):
    __tablename__ = 'dummy_two'
    
    id:             Mapped[int] = mapped_column(Integer, primary_key=True)
    name:           Mapped[str] = mapped_column(String(127))
    description:    Mapped[str]
    date:           Mapped[datetime]
    
    __table_args__ = {
        'doris_properties': {"replication_allocation": "tag.location.default: 1"},
        'comment': "Some table comment"
        }
    # doris_unique_key = 'id'
    # doris_distributed_by = HASH('id')
    # doris_distributed_by = RANDOM(32)
    # doris_partition_by = RANGE('id', ('PARTITION p1 VALUES LESS THAN ("1000")'))
    doris_autogen_primary_key = True


ROWS = [
    ('BMW', 'A car brand', datetime(2024, 1, 1)),
    ('Airbus', 'Construction bureau', datetime(2024, 2, 10)),
    ('Volvo', 'A car brand', datetime(2022, 12, 1, 10, 35))
]

def __mk_row(id :int, row: tuple[str, str, datetime]):
    return {
        'id': id,
        'name': row[0],
        'description': row[1],
        'date': row[2]
    }

if __name__ == '__main__':
    engine = make_eng()
    try:
        Dummy.drop(engine)
    except:
        print("Doesn't exist")
    Dummy.create(engine)
    # with Session(engine) as s:
    #     # print(engine.dialect.has_table(s.connection(), 'dummy_two'))
    #     vals = [__mk_row(i, ROWS[i]) for i in range(len(ROWS))]
    #     q = insert(Dummy)
    #     s.execute(q, vals)
    #     # s.execute(q)
    #     sel = select(Dummy)
    #     res = s.execute(sel)
    #     pprint(list(res))
    pass


#! /usr/bin/python3

# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

import logging
from typing import Any, Dict, List, Optional, Sequence
from sqlalchemy import Column, Table, log, exc, text
from sqlalchemy.dialects.mysql.base import MySQLDDLCompiler, MySQLIdentifierPreparer, MySQLDialect
from sqlalchemy.dialects.mysql import reflection as _reflection
from sqlalchemy.engine.interfaces import ReflectedTableComment, ReflectedForeignKeyConstraint
from sqlalchemy.dialects.mysql.mysqldb import MySQLDialect_mysqldb
from sqlalchemy.dialects.mysql.pymysql import MySQLDialect_pymysql
from sqlalchemy.engine import Connection
from sqlalchemy.util import topological

from doris_alchemy import datatype
from sqlalchemy.sql import sqltypes
# from sqlalchemy.sql.ddl import CreateTable
from sqlalchemy.schema import CreateTable, SchemaConst

from doris_alchemy.const import TABLE_KEY_OPTIONS, TABLE_PROPERTIES_SORT_TUPLES

from doris_alchemy.datatype import HASH, RANDOM, RenderedMixin
from doris_alchemy.util import format_properties, join_args_with_quote


logger = logging.getLogger(__name__)


class DorisDDLCompiler(MySQLDDLCompiler):
    def __init__(self, *args, **kw):
        super(DorisDDLCompiler, self).__init__(*args, **kw)
        self.dialect: DorisDialectMixin
    
    
    def visit_create_table(self, create: CreateTable, **kw):
        table: Table = create.element
        preparer = self.preparer

        text = "\nCREATE "
        if table._prefixes:
            text += " ".join(table._prefixes) + " "

        text += "TABLE "
        if create.if_not_exists:
            text += "IF NOT EXISTS "

        text += preparer.format_table(table) + " "

        create_table_suffix = self.create_table_suffix(table)
        if create_table_suffix:
            text += create_table_suffix + " "

        text += "("

        separator = "\n"

        # if only one primary key, specify it along with the column
        # first_pk = False primary key is not supported
        for create_column in create.columns:
            column = create_column.element
            # assert column.primary_key is False
            try:
                processed = self.process(create_column)
                if processed is not None:
                    text += separator
                    separator = ", \n"
                    text += "\t" + processed
                # if column.primary_key:
                #     first_pk = True
            except exc.CompileError as ce:
                raise exc.CompileError(
                    "(in table '%s', column '%s'): %s"
                    % (table.description, column.name, ce.args[0])
                ) from ce

        text += "\n)\n%s\n\n" % self.post_create_table(table)
        return text



    def __compile_table_arg(self, option: str, arg: Any, table_name: str) -> Optional[str]:
        option = option.replace("_", " ")
        # if option in _reflection._options_of_type_string:
        if option == 'COMMENT':
            arg = self.sql_compiler.render_literal_value(arg, sqltypes.String())
            return f'{option} {arg}'
        if option in TABLE_KEY_OPTIONS:
            if isinstance(arg, str):
                return '{} {}'.format(option, join_args_with_quote(arg))
            else:
                assert isinstance(arg, Sequence)
                return '{} {}'.format(option, join_args_with_quote(*arg))
        if option in ['PARTITION BY', 'DISTRIBUTED BY']:
            assert isinstance(arg, RenderedMixin)
            return '{} {}'.format(option, arg.render())
        if option == 'PROPERTIES':
            assert isinstance(arg, dict)
            return '{} {}'.format(option, format_properties(**arg))
        if option == 'ENGINE':
            assert isinstance(arg, str)
            return f'{option} {arg}'
        return None


    def post_create_table(self, table: Table):
        "Builds top level CREATE TABLE options, like ENGINE and COLLATE."

        table_opts = []
        opts = {}
        for k, v in table.kwargs.items():
            if k.startswith("%s_" % self.dialect.name):
                opts[k[len(self.dialect.name) + 1:].upper()] = v
        if table.comment is not None:
            opts["COMMENT"] = table.comment
        pk = table.primary_key
        if opts.get('AUTOGEN_PRIMARY_KEY', False) and len(pk) > 0:
            pk_key = tuple(c.name for c in pk)
            if 'UNIQUE_KEY' not in opts:
                opts['UNIQUE_KEY'] = pk_key
            if 'DISTRIBUTED_BY' not in opts:
                opts['DISTRIBUTED_BY'] = HASH(pk_key)
                
            
        sorted_opts = topological.sort(TABLE_PROPERTIES_SORT_TUPLES, opts)
        for opt in sorted_opts:
            __arg = opts[opt]
            __compiled = self.__compile_table_arg(opt, __arg, table.name)
            if __compiled:
                table_opts.append(__compiled)
        return "\n".join(table_opts)
    
    def visit_create_column(self, create, first_pk=False, **kw):
        column = create.element
        if column.system:
            return None
        text = self.get_column_specification(column, first_pk=first_pk)
        const = " ".join(
            self.process(constraint) for constraint in column.constraints
        )
        if const:
            text += " " + const
        return text
    
    def get_column_specification(self, column: Column, **kw):
        """Builds column DDL."""
        if (
            self.dialect.is_mariadb is True
            and column.computed is not None
            and column._user_defined_nullable is SchemaConst.NULL_UNSPECIFIED
        ):
            column.nullable = True
        colspec = [
            self.preparer.format_column(column),
            self.dialect.type_compiler_instance.process(
                column.type, type_expression=column
            ),
        ]

        if column.computed is not None:
            colspec.append(self.process(column.computed))

        is_timestamp = isinstance(
            column.type._unwrapped_dialect_impl(self.dialect),
            sqltypes.TIMESTAMP,
        )

        if not column.nullable:
            colspec.append("NOT NULL")

        # see: https://docs.sqlalchemy.org/en/latest/dialects/mysql.html#mysql_timestamp_null  # noqa
        elif column.nullable and is_timestamp:
            colspec.append("NULL")

        comment = column.comment
        if comment is not None:
            literal = self.sql_compiler.render_literal_value(
                comment, sqltypes.String()
            )
            colspec.append("COMMENT " + literal)

        else:
            default = self.get_column_default_string(column)
            if default is not None:
                colspec.append("DEFAULT " + default)
        return " ".join(colspec)


@log.class_logger
class DorisDialectMixin(MySQLDialect, log.Identified):
    # Caching
    # Warnings are generated by SQLAlchmey if this flag is not explicitly set
    # and tests are needed before being enabled
    supports_statement_cache = False


    name = 'doris'
    preparer = MySQLIdentifierPreparer

    def __init__(self, *args, **kw):
        super(DorisDialectMixin, self).__init__(*args, **kw)
        self.preparer = MySQLIdentifierPreparer
        self.identifier_preparer: MySQLIdentifierPreparer = self.preparer(self)

    def has_table(self, connection: Connection, table_name: str, schema=None, **kw) -> bool:
        self._ensure_has_table_connection(connection)

        if schema is None:
            schema = self.default_schema_name

        assert schema is not None

        full_name = ".".join(
            self.identifier_preparer._quote_free_identifiers(
                schema, table_name
            )
        )

        # DESCRIBE *must* be used because there is no information schema
        # table that returns information on temp tables that is consistently
        # available on MariaDB / MySQL / engine-agnostic etc.
        # therefore we have no choice but to use DESCRIBE and an error catch
        # to detect "False".  See issue #9058

        try:
            with connection.exec_driver_sql(
                f"DESCRIBE {full_name}",
                execution_options={"skip_user_error_events": True},
            ) as rs:
                return rs.fetchone() is not None
        except exc.DBAPIError as e:
            # https://dev.mysql.com/doc/mysql-errors/8.0/en/server-error-reference.html  # noqa: E501
            # there are a lot of codes that *may* pop up here at some point
            # but we continue to be fairly conservative.  We include:
            # 1146: Table '%s.%s' doesn't exist - what every MySQL has emitted
            # for decades
            #
            # mysql 8 suddenly started emitting:
            # 1049: Unknown database '%s'  - for nonexistent schema
            #
            # also added:
            # 1051: Unknown table '%s' - not known to emit
            #
            # there's more "doesn't exist" kinds of messages but they are
            # less clear if mysql 8 would suddenly start using one of those
            # print('caught exception', e)
            if self._extract_error_code(e.orig) in (1105, 1051):
                if e.orig is None:
                    return False
                info: str = e.orig.args[1].split('detailMessage = ')[-1]
                if info.startswith('Unknown table'):
                    return False
            raise


    def get_schema_names(self, connection: Connection, **kw):
        rp = connection.exec_driver_sql("SHOW schemas")
        return [r[0] for r in rp]

    def get_table_names(self, connection: Connection, schema: Optional[str]=None, **kw):
        """Return a Unicode SHOW TABLES from a given schema."""
        if schema is not None:
            current_schema = schema
        else:
            current_schema = self.default_schema_name
        assert current_schema, 'Failed to establish current schema'

        charset = self._connection_charset

        rp = connection.exec_driver_sql(
            "SHOW FULL TABLES FROM %s"
            % self.identifier_preparer.quote_identifier(current_schema)
        )

        return [
            row[0]
            for row in self._compat_fetchall(rp, charset=charset)
        ]

    def get_view_names(self, connection, schema: Optional[str]=None, **kw):
        if schema is None:
            schema = self.default_schema_name
        assert schema, 'Failed to get schema name'
        charset = self._connection_charset
        rp = connection.exec_driver_sql(
            "SHOW FULL TABLES FROM %s"
            % self.identifier_preparer.quote_identifier(schema)
        )
        return [
            row[0]
            for row in self._compat_fetchall(rp, charset=charset)
            if row[1] in ("VIEW", "SYSTEM VIEW")
        ]

    def get_columns(self, connection: Connection, table_name: str, schema: Optional[str] = None, **kw) -> List[Dict[str, Any]]:
        if not self.has_table(connection, table_name, schema):
            raise exc.NoSuchTableError(f"schema={schema}, table={table_name}")
        schema = schema or self._get_default_schema_name(connection)

        quote = self.identifier_preparer.quote_identifier
        full_name = quote(table_name)
        if schema:
            full_name = "{}.{}".format(quote(schema), full_name)

        res = connection.execute(text(f"SHOW COLUMNS FROM {full_name}"))
        columns = []
        for record in res:
            column = dict(
                name=record.Field,
                type=datatype.parse_sqltype(record.Type),
                nullable=record.Null == "YES",
                default=record.Default,
            )
            columns.append(column)
        return columns


    def get_pk_constraint(self, connection, table_name, schema=None, **kw):
        return {  # type: ignore  # pep-655 not supported
            "name": None,
            "constrained_columns": [],
        }

    def get_unique_constraints(
            self, connection: Connection, table_name: str, schema: Optional[str] = None, **kw
    ) -> List[Dict[str, Any]]:
        return []

    def get_check_constraints(
            self, connection: Connection, table_name: str, schema: Optional[str] = None, **kw
    ) -> List[Dict[str, Any]]:
        return []

    def get_foreign_keys(
            self, connection: Connection, table_name: str, schema: Optional[str] = None, **kw
    ) -> List[ReflectedForeignKeyConstraint]:
        return []

    def get_primary_keys(self, connection: Connection, table_name: str, schema: Optional[str] = None, **kw) -> List[str]:
        pk = self.get_pk_constraint(connection, table_name, schema)
        return pk.get("constrained_columns")  # type: ignore

    def get_indexes(self, connection, table_name, schema=None, **kw):
        return []

    def has_sequence(self, connection: Connection, sequence_name: str, schema: Optional[str] = None, **kw) -> bool:
        return False

    def get_sequence_names(self, connection: Connection, schema: Optional[str] = None, **kw) -> List[str]:
        return []

    def get_temp_view_names(self, connection: Connection, schema: Optional[str] = None, **kw) -> List[str]:
        return []

    def get_temp_table_names(self, connection: Connection, schema: Optional[str] = None, **kw) -> List[str]:
        return []

    def get_table_options(self, connection, table_name, schema=None, **kw):
        return {}

    def get_table_comment(self, connection: Connection, table_name: str, schema: Optional[str] = None, **kw) -> ReflectedTableComment:
        return ReflectedTableComment(text=None)


class DorisDialect_pymysql(DorisDialectMixin, MySQLDialect_pymysql): # type: ignore
    supports_statement_cache = False
    ddl_compiler = DorisDDLCompiler


class DorisDialect_mysqldb(DorisDialectMixin, MySQLDialect_mysqldb): # type: ignore
    supports_statement_cache = False
    ddl_compiler = DorisDDLCompiler


try:
    # using MySQLdb as default driver if available
    import MySQLdb
    DorisDialect = DorisDialect_mysqldb

except ModuleNotFoundError:
    DorisDialect = DorisDialect_pymysql



from typing import TypeAlias


TABLE_PROPERTIES_SORT_TUPLES = [
    ("UNIQUE_KEY", "PARTITION_BY"),
    ("AGGREGATE_KEY", "PARTITION_BY"),
    ("DUPLICATE_KEY", "PARTITION_BY"),
    ("UNIQUE_KEY",    "DISTRIBUTED_BY"),
    ("AGGREGATE_KEY", "DISTRIBUTED_BY"),
    ("DUPLICATE_KEY", "DISTRIBUTED_BY"),
    ("PARTITION_BY", "DISTRIBUTED_BY"),
    ("DISTRIBUTED_BY", "PROPERTIES"),
    ("UNIQUE_KEY", "COMMENT"),
    ("AGGREGATE_KEY", "COMMENT"),
    ("DUPLICATE_KEY", "COMMENT"),
    ('COMMENT', 'PARTITION_BY'),
    ('COMMENT', 'DISTRIBUTED_BY')
]

TABLE_KEY_OPTIONS = (
    "UNIQUE KEY",
    "AGGREGATE KEY",
    "DUPLICATE KEY",
)


from typing import Any, Sequence


def join_args_with_quote(*args):
    args = [f'`{a}`' for a in args]
    args_str = ', '.join(args)
    return '(' + args_str + ')'


def format_properties(**kwargs):
    entries = []
    for k, v in kwargs.items():
        entry = f'"{k}" = "{v}",'
        entries.append(entry)
    result_str = '\n    '.join(entries)
    return '(\n    ' + result_str[:-1] + '\n)'


def ensure_sequence(value: Any) -> Sequence:
    if isinstance(value, str):
        return [value]
    if isinstance(value, list|tuple):
        return value
    return [value]